"""
Tests for the extra scripts.
"""
from __future__ import absolute_import, unicode_literals

from django.test import TestCase
from mock import patch
from path import Path
from testfixtures import LogCapture

from eox_theming.theming.extra_scripts import process_scripts, validate_script_attributes


class TestsProcessExtraScripts(TestCase):
    """ Tests for extra scripts"""

    @patch('eox_theming.configuration.ThemingConfiguration.options')
    def test_returned_process_scripts(self, themingConfig_mock):
        """
        Test process_scripts function returns a dictionary with all the valid scripts
        in the correct order.
        """
        path = Path("/test")
        themingConfig_mock.return_value = {
            ".*/test": [
                {
                    "src": "https://www.test-external-script.com/js/myScript1.js",
                    "type": "external",
                    "media_type": "module"
                },
                {
                    "content": "alert('This alert box was called with the onload event');",
                    "type": "inline"
                }
            ]
        }
        test_scripts = [
            {
                "src": "https://www.test-external-script.com/js/myScript1.js",
                "type": "external",
                "media_type": "module"
            },
            {
                "content": "alert('This alert box was called with the onload event');",
                "type": "inline",
                "media_type": "text/javascript"
            },
        ]

        scripts = process_scripts(path)

        self.assertEqual(test_scripts, scripts)

    @patch('eox_theming.configuration.ThemingConfiguration')
    def test_returned_(self, themingConfig_mock):
        """
        Test process_scripts function returns only the scripts that have a valid configuration.
        """
        path = Path("/test")
        themingConfig_mock.return_value = {
            ".*/test": [
                {
                    "content": "alert('This alert box was called with the onload event');",
                    "type": "src"
                }
            ]
        }

        scripts = process_scripts(path)

        self.assertEqual(None, scripts)

    @patch('eox_theming.configuration.ThemingConfiguration')
    def test_returned_path_scripts(self, themingConfig_mock):
        """
        Test process_scripts function returns only the scripts that match the current request path.
        """
        path = Path("/test")
        themingConfig_mock.return_value = {
            ".*/dashboard": [
                {
                    "src": "https://www.test-external-script.com/js/myScript1.js",
                    "type": "external",
                    "media_type": "text/javascript"
                }
            ]
        }

        scripts = process_scripts(path)

        self.assertEqual(None, scripts)

    def test_validate_script_attributes_fails_silently(self):
        """
        Test validate_script_attributes function logs error when a script has a missing/incorrect attribute.
        """
        values = {
            "type": "external",
            "media_type": "text/javascript"
        }
        log_message = "Script could not get loaded. 'src' attribute is missing."

        with LogCapture() as log:
            validate_script_attributes(values)
            log.check(("eox_theming.theming.extra_scripts",
                       "ERROR",
                       log_message))



"""
This stub module contains an empty extension point to modify the template loading
This module's logic applies both to Mako and Django Templates
"""
from eox_theming.configuration import ThemingConfiguration
from eox_theming.edxapp_wrapper.loaders import get_openedx_theme_loader, get_theme_filesystem_loader
from eox_theming.edxapp_wrapper.mako import get_mako_loader

MakoLoader = get_mako_loader()
OpenedxThemeLoader = get_openedx_theme_loader()
ThemeFilesystemLoader = get_theme_filesystem_loader()


class EoxThemeTemplateLoader(OpenedxThemeLoader):
    """
    Overrided to use EoxThemeFilesystemLoader instead of platform ThemeFilesystemLoader.

    See: openedx.core.djangoapps.theming.template_loaders.py
    """
    def __init__(self, *args):
        MakoLoader.__init__(self, EoxThemeFilesystemLoader(*args))


class EoxThemeFilesystemLoader(ThemeFilesystemLoader):
    """
    Overrode to append current theme, parent, and grandparent theme dirs so templates are looked up
    in the correct order.
    """

    @staticmethod
    def get_theme_template_sources():
        """
        Return template sources for the given theme and if request object is None (this would be the case for
        management commands) return template sources for all themes.
        """
        if not ThemingConfiguration.theming_helpers.get_current_request():
            # if request object is not present, then this method is being called inside a management
            # command and return all theme template sources for compression
            return ThemingConfiguration.theming_helpers.get_all_theme_template_dirs()
        else:
            # template is being accessed by a view, so return templates sources for current theme
            template_dirs = list()
            theme = ThemingConfiguration.theming_helpers.get_current_theme()
            if theme:
                template_dirs = theme.template_dirs

                parent_theme_dirs = EoxThemeFilesystemLoader.get_parent_theme_template_sources()
                grandparent_theme_dirs = EoxThemeFilesystemLoader.get_grandparent_theme_template_sources()
                # append parent and grandparent dirs to the end so templates are looked up in the correct order
                if isinstance(parent_theme_dirs, list) and isinstance(grandparent_theme_dirs, list):
                    template_dirs += parent_theme_dirs + grandparent_theme_dirs

            return template_dirs

    @staticmethod
    def get_parent_theme_template_sources():
        """
        Return the template dirs of the parent theme.
        """
        default_theme = ThemingConfiguration.get_parent_or_default_theme()
        template_paths = list()
        if default_theme:
            return default_theme.template_dirs

        return template_paths

    @staticmethod
    def get_grandparent_theme_template_sources():
        """
        Return the template dirs of the grandparent theme.
        """
        grandparent_theme = None
        grandparent_name = ThemingConfiguration.options('theme', 'grandparent', default=None)
        if grandparent_name:
            grandparent_theme = ThemingConfiguration.get_wrapped_theme(grandparent_name)

        template_paths = list()
        if grandparent_theme:
            return grandparent_theme.template_dirs

        return template_paths



"""
Set up lookup paths for mako templates.

See: common.djangoapps.edxmako.paths
"""

import os
import re

import pkg_resources
from django.conf import settings
from mako.exceptions import TopLevelLookupException

from eox_theming.configuration import ThemingConfiguration
from eox_theming.edxapp_wrapper.mako import (
    get_clear_lookups,
    get_dynamictemplate_lookup,
    get_lookup,
    get_top_level_template_uri,
)

DynamicTemplateLookup = get_dynamictemplate_lookup()
TopLevelTemplateURI = get_top_level_template_uri()
LOOKUP = get_lookup()
clear_lookups = get_clear_lookups()


class EoxDynamicTemplateLookup(DynamicTemplateLookup):
    """
    The eox theme EoxDynamicTemplateLookup is the intervention point to make sure
    that we are loading the templates from the request theme and the default site theme.

    See: common.djangoapps.edxmako.paths
    """
    def adjust_uri(self, uri, calling_uri):
        """
        This method was reimplemented to use eox-theming strip_site_theme_templates_path and
        get_template_path_with_theme instead of the equivalent functions used in the platform.
        """
        # Make requested uri relative to the calling uri.
        relative_uri = super(EoxDynamicTemplateLookup, self).adjust_uri(uri, calling_uri)
        adjusted_uri = get_template_path_with_theme(strip_site_theme_templates_path(relative_uri))
        # Is the calling template (calling_uri) which is including or inheriting current template (uri)
        # located inside a theme?
        if calling_uri != strip_site_theme_templates_path(calling_uri):
            # Is the calling template trying to include/inherit itself?
            if calling_uri == adjusted_uri:
                return TopLevelTemplateURI(relative_uri)
        return adjusted_uri

    def get_template(self, uri):
        """
        Overridden method for using get_template_path_with_theme from eox-theming instead of the function used in
        the platform.
        """

        if isinstance(uri, TopLevelTemplateURI):
            template = self._get_toplevel_template(uri)
        else:
            try:
                # Try to find themed template, i.e. see if current theme overrides the template
                template = super(EoxDynamicTemplateLookup, self).get_template(get_template_path_with_theme(uri))
            except TopLevelLookupException:
                template = self._get_toplevel_template(uri)

        return template

    def _get_toplevel_template(self, uri):
        """
        Lookup a default/toplevel template, ignoring current theme.
        """
        # Strip off the prefix path to request theme and parent theme and look in default template dirs.
        return super(EoxDynamicTemplateLookup, self)._get_toplevel_template(strip_site_theme_templates_path(uri))


def add_lookup(namespace, directory, package=None, prepend=False):
    """
    The difference with the edxmako implementation is that this uses the EoxDynamicTemplateLookup
    instead of DynamicTemplateLookup.

    See: common.djangoapps.edxmako.paths

    """
    templates = LOOKUP.get(namespace)
    if not templates:
        LOOKUP[namespace] = templates = EoxDynamicTemplateLookup(
            module_directory=settings.MAKO_MODULE_DIR,
            output_encoding='utf-8',
            input_encoding='utf-8',
            default_filters=['decode.utf8'],
            encoding_errors='replace',
        )
    if package:
        directory = pkg_resources.resource_filename(package, directory)
    templates.add_directory(directory, prepend=prepend)


def get_template_path_with_theme(relative_path):
    """
    The change with respect of the edx-platform version is that in this case, the templates are
    not searched only on the current site theme. They are also searched in the parent(default) theme.

    See: openedx.core.djangoapps.theming.helpers

    Example:
        >> get_template_path_with_theme('header.html')
        '/red-theme/lms/templates/header.html'

    Parameters:
        relative_path (str): template's path relative to the templates directory e.g. 'footer.html'

    Returns:
        (str): template path in current site's theme
    """
    relative_path = os.path.normpath(relative_path)
    template_name = re.sub(r'^/+', '', relative_path)

    theme = ThemingConfiguration.theming_helpers.get_current_theme()
    parent_theme = ThemingConfiguration.get_parent_or_default_theme()

    # Try with the theme.name
    if theme:
        # strip `/` if present at the start of relative_path
        template_path = theme.template_path / template_name
        absolute_path = theme.path / "templates" / template_name
        if absolute_path.exists():
            return str(template_path)

        if not parent_theme or theme.name == parent_theme.name:
            return relative_path

    if not parent_theme:
        return relative_path

    # Try with the theme.parent site theme
    template_path = parent_theme.template_path / template_name
    absolute_path = parent_theme.path / "templates" / template_name

    if absolute_path.exists():
        return str(template_path)

    # Try with grandparent
    grandparent_name = ThemingConfiguration.options('theme', 'grandparent', default=None)
    if grandparent_name:
        grandparent_theme = ThemingConfiguration.get_wrapped_theme(grandparent_name)
        template_path = grandparent_theme.template_path / template_name
        absolute_path = grandparent_theme.path / "templates" / template_name

        if absolute_path.exists():
            return str(template_path)

    return relative_path


def strip_site_theme_templates_path(uri):
    """
    The change with respect of the edx-platform version is that in this case, are stripped the
    theme and parent theme from the uri

    See: openedx.core.djangoapps.theming.helpers

    Example:
        >> strip_site_theme_templates_path('/red-theme/lms/templates/header.html')
        'header.html'

    Arguments:
        uri (str): template path from which to remove site theme path. e.g. '/red-theme/lms/templates/header.html'

    Returns:
        (str): template path with site theme path removed.
    """
    theme = ThemingConfiguration.theming_helpers.get_current_theme()
    parent_theme = ThemingConfiguration.get_parent_or_default_theme()

    if theme:
        templates_path = "/".join([
            theme.theme_dir_name,
            ThemingConfiguration.theming_helpers.get_project_root_name(),
            "templates"
        ])

        uri = re.sub(r'^/*' + templates_path + '/*', '', uri)

    if not parent_theme:
        return uri

    # Do the same with the parent theme
    templates_path = "/".join([
        parent_theme.theme_dir_name,
        ThemingConfiguration.theming_helpers.get_project_root_name(),
        "templates"
    ])

    uri = re.sub(r'^/*' + templates_path + '/*', '', uri)

    grandparent_name = ThemingConfiguration.options('theme', 'grandparent', default=None)
    if grandparent_name:
        grandparent_theme = ThemingConfiguration.get_wrapped_theme(grandparent_name)
        if not grandparent_theme:
            return uri

        # Do the same with the grandparent theme
        templates_path = "/".join([
            grandparent_theme.theme_dir_name,
            ThemingConfiguration.theming_helpers.get_project_root_name(),
            "templates"
        ])

        uri = re.sub(r'^/*' + templates_path + '/*', '', uri)

    return uri



"""
This module abstracts the storage defined in the platform
"""
from importlib import import_module

from django.conf import settings


def get_theme_storage():
    """ Get the Open edX Theme Finder class. """
    backend = import_module(settings.EOX_THEMING_STORAGE_BACKEND)
    return backend.get_theme_storage()


def get_themecached_mixin():
    """Return the ThemeCached mixin when called during runtime"""
    backend = import_module(settings.EOX_THEMING_STORAGE_BACKEND)
    return backend.get_themecached_mixin()


def get_themepipeline_mixin():
    """Return the Theme pipeline mixin when called during runtime"""
    backend = import_module(settings.EOX_THEMING_STORAGE_BACKEND)
    return backend.get_themepipeline_mixin()


def get_pipeline_forgiving_storage():
    """
    Return the PipelineForgiving storage when called during runtime
    No longer used on juniper.
    """
    backend = import_module(settings.EOX_THEMING_STORAGE_BACKEND)
    return backend.get_pipeline_forgiving_storage()


def get_production_mixin():
    """Return the Production storage mixin when called during runtime"""
    backend = import_module(settings.EOX_THEMING_STORAGE_BACKEND)
    return backend.get_production_mixin()



"""
Simple backend that returns the platform's ThemeTemplateLoader class
"""
from openedx.core.djangoapps.theming.template_loaders import (  # pylint: disable=import-error
    ThemeFilesystemLoader,
    ThemeTemplateLoader,
)


def get_openedx_theme_loader():
    """Return the template loader class when called during runtime"""
    return ThemeTemplateLoader


def get_theme_filesystem_loader():
    """Return the filesystem loader class when called during runtime"""
    return ThemeFilesystemLoader



"""
This module abstracts the source functions from the platform or plugins code
to prevent a dependency requirement while testing.

Some functions are so simple they do not require a backend.
"""
import logging
import operator
# forward compatibility for Python 3
from functools import reduce  # pylint: disable=redefined-builtin, useless-suppression

from django.conf import settings

from eox_theming.edxapp_wrapper.configuration_helpers import get_configuration_helper

configuration_helpers = get_configuration_helper()
LOG = logging.getLogger(__name__)


def from_site_config(*args):
    """
    This source is compatible with the site_configurations from Open edX platform.
    """
    options_dict = configuration_helpers.get_value("THEME_OPTIONS", {})
    if args:
        try:
            value = reduce(operator.getitem, args, options_dict)
        except (AttributeError, KeyError):
            LOG.debug("Found nothing when reading the theme options for %s", ".".join(args))
            value = None
    return value


def from_eox_tenant_config_theming(*args):  # pylint: disable=unused-argument
    """
    This source is the most modern way of reading the theme configuration.
    Fully dependant on an advanced use of eox-tenant.
    """
    LOG.debug("Not implemented: from_eox_tenant_config_theming")


def from_eox_tenant_microsite_v0(*args):
    """
    This source must act as a compatibility layer with the ungrouped way of storing
    config variables from the first iteration of the bragi theme.
    """
    key = args[-1]
    return configuration_helpers.get_value(key, None)


def from_eox_tenant_config_lms(*args):
    """
    Given that the eox_microsite version loads all the config variables into
    the django.conf.settings object, we can defer to the django_settings source.
    """
    return from_django_settings(*args)


def from_eox_tenant_microsite_v1(*args):
    """
    Given that the eox_microsite version loads all the config variables into
    the django.conf.settings object, we can defer to the django_settings source.
    """
    return from_django_settings(*args)


def from_django_settings(*args):
    """
    Takes a THEME_OPTIONS dictionary in the settings module as the source.
    """
    options_dict = getattr(settings, "THEME_OPTIONS", {})

    if args:
        try:
            value = reduce(operator.getitem, args, options_dict)
        except (AttributeError, KeyError):
            LOG.debug("Found nothing when reading the theme options for %s on django settings", ".".join(args))
            value = None
    return value



# -*- coding: utf-8 -*-
# -*- coding: utf-8 -*-
"""
Tests for the selection middleware
"""
from __future__ import absolute_import, unicode_literals

from django.test import RequestFactory, TestCase
from mock import mock, patch


class TestsEoxThemeMiddleware(TestCase):
    """ Tests for the eox_theming theme selection middleware """

    @patch('eox_theming.edxapp_wrapper.models.get_openedx_site_theme_model')
    def test_theme_selected(self, module_mock):
        """ The first implementation must return a site_theme in the requests object """
        request = RequestFactory().get('/')
        site_theme = mock.Mock()
        module_mock.return_value.return_value = site_theme

        from eox_theming.theming.middleware import EoxThemeMiddleware  # pylint: disable=import-outside-toplevel
        EoxThemeMiddleware().process_request(request)

        self.assertEqual(site_theme, request.site_theme)  # pylint: disable=no-member



""" Backend abstraction for edxmako used in tests. """


def get_mako_loader():
    """ Get MakoLoader. """
    return object


def get_dynamictemplate_lookup():
    """ Abstraction for tests. """
    return object


def get_top_level_template_uri():
    """ Abstraction for tests. """
    return object


def get_lookup():
    """ Abstraction for tests. """
    return {}


def get_clear_lookups():
    """ Abstraction for tests. """
    def mock_function():
        """function for the tests"""
        return
    return mock_function



"""
Simple backend that returns the platform's SiteTheme model
"""
from openedx.core.djangoapps.theming.models import SiteTheme  # pylint: disable=import-error


def get_openedx_site_theme_model():
    """Return the model class when called during runtime"""
    return SiteTheme



# -*- coding: utf-8 -*-
# -*- coding: utf-8 -*-
"""
Tests for eox theming paths
"""
from __future__ import absolute_import, unicode_literals

from django.test import TestCase
from mock import mock, patch
from path import Path

from eox_theming.theming.paths import get_template_path_with_theme, strip_site_theme_templates_path


class ThemingPathsTest(TestCase):
    """ Tests for the eox_theming paths module """

    @patch('eox_theming.configuration.ThemingConfiguration.theming_helpers')
    @patch('eox_theming.configuration.ThemingConfiguration.get_parent_or_default_theme')
    def test_strip_site_theme_from_uri(self, parent_mock, helper_mock):
        """
        Test site theme templates path is stripped from the given template path.
        """
        theme = mock.Mock()
        theme.theme_dir_name = 'bragi'
        helper_mock.get_current_theme.return_value = theme
        helper_mock.get_project_root_name.return_value = 'lms'

        default_theme = mock.Mock()
        default_theme.theme_dir_name = 'default-theme'
        default_theme.name = 'default_theme'
        default_theme.path = Path('/themes/default_theme/lms')
        default_theme.template_path = Path('default-theme/lms/templates')

        parent_mock.return_value = default_theme
        template_path = strip_site_theme_templates_path('bragi/lms/templates/header.html')
        self.assertEqual(template_path, 'header.html')

    @patch('eox_theming.configuration.ThemingConfiguration.theming_helpers')
    @patch('eox_theming.configuration.ThemingConfiguration.get_parent_or_default_theme')
    def test_strip_site_default_theme_from_uri(self, parent_mock, helper_mock):
        """
        Test default site theme templates path is stripped from the given template path.
        """
        theme = mock.Mock()
        theme.theme_dir_name = 'bragi'
        theme.name = 'bragi'
        helper_mock.get_current_theme.return_value = theme
        helper_mock.get_project_root_name.return_value = 'lms'

        default_theme = mock.Mock()
        default_theme.theme_dir_name = 'default-theme'
        default_theme.name = 'default_theme'
        default_theme.path = Path('/themes/default_theme/lms')
        default_theme.template_path = Path('default-theme/lms/templates')

        parent_mock.return_value = default_theme
        template_path = strip_site_theme_templates_path('default-theme/lms/templates/header.html')
        self.assertEqual(template_path, 'header.html')

        template_path = strip_site_theme_templates_path('/theme-name/lms/templates/header.html')
        self.assertNotEqual(template_path, 'header.html')

    @patch('os.path.exists')
    @patch('eox_theming.configuration.ThemingConfiguration.theming_helpers')
    @patch('eox_theming.configuration.Theme')
    def test_get_template_path_with_theme(self, theme_mock, helper_mock, os_mock):
        """
        Tests template paths are returned from enabled theme.
        """
        os_mock.return_value = True

        theme = mock.Mock()
        theme.theme_dir_name = 'bragi'
        theme.path = Path('/themes/bragi/lms')
        theme.template_path = Path('bragi/lms/templates')
        helper_mock.get_current_theme.return_value = theme
        helper_mock.get_project_root_name.return_value = 'lms'

        template_path = get_template_path_with_theme('header.html')

        theme_mock.assert_called_once()
        self.assertEqual(template_path, 'bragi/lms/templates/header.html')

    @patch('os.path.exists')
    @patch('eox_theming.configuration.ThemingConfiguration.theming_helpers')
    @patch('eox_theming.configuration.ThemingConfiguration.get_parent_or_default_theme')
    def test_get_template_with_parent_theme(self, parent_mock, helper_mock, os_mock):
        """
        Tests parent theme template paths are returned if template is not found in the theme.
        """
        os_mock.side_effect = [False, True]
        theme = mock.Mock()
        theme.theme_dir_name = 'bragi'
        theme.name = 'bragi'
        theme.path = Path('/themes/bragi/lms')
        theme.template_path = Path('bragi/lms/templates')

        default_theme = mock.Mock()
        default_theme.theme_dir_name = 'default-theme'
        default_theme.name = 'default_theme'
        default_theme.path = Path('/themes/default_theme/lms')
        default_theme.template_path = Path('default-theme/lms/templates')

        parent_mock.return_value = default_theme

        helper_mock.get_current_theme.return_value = theme
        helper_mock.get_project_root_name.return_value = 'lms'

        template_path = get_template_path_with_theme('header.html')
        self.assertEqual(template_path, 'default-theme/lms/templates/header.html')

    @patch('os.path.exists')
    @patch('eox_theming.configuration.ThemingConfiguration.theming_helpers')
    @patch('eox_theming.configuration.ThemingConfiguration.get_parent_or_default_theme')
    def test_get_template_not_found_in_themes(self, parent_mock, helper_mock, os_mock):
        """
        Tests default template paths are returned if template is not found in the theme.
        """
        os_mock.side_effect = [False, False]
        theme = mock.Mock()
        theme.theme_dir_name = 'bragi'
        theme.name = 'bragi'
        theme.path = Path('/themes/bragi/lms')
        theme.template_path = Path('bragi/lms/templates')

        default_theme = mock.Mock()
        default_theme.theme_dir_name = 'default-theme'
        default_theme.name = 'default_theme'
        default_theme.path = Path('/themes/default_theme/lms')
        default_theme.template_path = Path('default-theme/lms/templates')

        parent_mock.return_value = default_theme
        helper_mock.get_current_theme.return_value = theme
        helper_mock.get_project_root_name.return_value = 'lms'

        template_path = get_template_path_with_theme('header.html')
        self.assertEqual(template_path, 'header.html')



"""
Static file finders for Django.

This is the original note from the edx-platform code:
https://docs.djangoproject.com/en/1.8/ref/settings/#std:setting-STATICFILES_FINDERS
Yes, this interface is private and undocumented, but we need to access it anyway.

In order to deploy Open edX in production, it's important to be able to collect
and process static assets: images, CSS, JS, fonts, etc. Django's collectstatic
system is the accepted way to do that in Django-based projects, but that system
doesn't handle every kind of collection and processing that web developers need.
Other open source projects like `Django-Pipeline`_ and `Django-Require`_ hook
into Django's collectstatic system to provide features like minification,
compression, Sass pre-processing, and require.js optimization for assets before
they are pushed to production. To make sure that themed assets are collected
and served by the system (in addition to core assets), we need to extend this
interface, as well.

.. _Django-Pipeline: https://django-pipeline.readthedocs.org/
.. _Django-Require: https://github.com/etianen/django-require
"""
from eox_theming.edxapp_wrapper.finders import get_openedx_theme_finder

OpenedxThemeFinder = get_openedx_theme_finder()


class EoxThemeFilesFinder(OpenedxThemeFinder):
    """
    The eox theme files finder is the intervention point to make sure
    that we are loading all the themes statics assets into the platform.

    See: openedx/core/djangoapps/theming/finders.py
    """



"""Backend abstraction for edxmako. """

from edxmako import LOOKUP, clear_lookups  # pylint: disable=import-error
from edxmako.makoloader import MakoLoader  # pylint: disable=import-error
from edxmako.paths import DynamicTemplateLookup, TopLevelTemplateURI  # pylint: disable=import-error


def get_mako_loader():
    """ Get MakoLoader. """
    return MakoLoader


def get_dynamictemplate_lookup():
    """ Get DynamicTemplateLookup. """
    return DynamicTemplateLookup


def get_top_level_template_uri():
    """ Get TopLevelTemplateURI"""
    return TopLevelTemplateURI


def get_lookup():
    """ Get edxmako LOOKUP dict"""
    return LOOKUP


def get_clear_lookups():
    """ Get clear_lookups function """
    return clear_lookups



"""
Template tags and helper functions for displaying breadcrumbs in page titles
based on the current micro site.
"""
from __future__ import absolute_import, unicode_literals

from ast import literal_eval

from django import template

from eox_theming.configuration import ThemingConfiguration

register = template.Library()


class ThemingOptionsNode(template.Node):
    """
    Django templates nodes to render the advanced tag.
    Acts as a wrapper to the ThemingConfiguration.options call.
    """
    def __init__(self, args, default):
        self.options_args = args
        self.options_default = default

    def render(self, context):
        return ThemingConfiguration.options(*self.options_args, default=self.options_default)


@register.tag(name="theming_options")
def do_options_call(parser, token):  # pylint: disable=unused-argument
    """
    Django templates tag definition.
    """
    contents = token.split_contents()
    args = contents[1:]

    parsed_args = []
    default = None

    for arg in args:
        try:
            val = literal_eval(arg)
        except (SyntaxError, ValueError):
            val = arg

        try:
            if val.startswith('default='):
                default = val[8:]
            else:
                parsed_args.append(arg)
        except AttributeError:
            parsed_args.append(arg)

    return ThemingOptionsNode(parsed_args, default=default)



"""
This context processor is added to every request to render a mako or django template

It can be used using the theming.options(args) helper.
"""
from eox_theming.configuration import ThemingConfiguration
from eox_theming.theming.extra_scripts import process_scripts


def eox_configuration(request):
    """
    It inserts a theming object with the options helper function
    This is the last processor to be run.
    """
    return {
        'scripts': process_scripts(request.path_info),
        'theming': ThemingConfiguration,
    }



# -*- coding: utf-8 -*-
"""
Tests for the ThemingConfiguration class
"""
from __future__ import absolute_import, unicode_literals

from django.test import TestCase
from django.test.utils import override_settings
from mock import patch

from eox_theming.configuration import ThemingConfiguration


class TestThemingConfiguration(TestCase):
    """ Tests for the ThemingConfiguration class """

    def test_options_call_default(self):
        """ Calling options must return the default """
        result = ThemingConfiguration.options('section', 'item', default='Unique_string')
        self.assertEqual('Unique_string', result)

    @override_settings(THEME_OPTIONS={'section': {'item': 'Given_String'}})
    def test_options_call_exists_django_settings(self):
        """ Calling options must return the real value  """
        result = ThemingConfiguration.options('section', 'item', default='Unique_string')
        self.assertEqual('Given_String', result)

    @override_settings(THEME_OPTIONS=[])
    def test_options_call_error(self):
        """ Calling options with error data must return the Default """
        result = ThemingConfiguration.options('section', 'item', default='Unique_string')
        self.assertEqual('Unique_string', result)

    @override_settings(THEME_OPTIONS={'section': {'item': False}})
    def test_falsy_value(self):
        """ Calling options must return the value defined in the source even if that is false """
        result = ThemingConfiguration.options('section', 'item', default=None)
        self.assertIsNotNone(result)
        self.assertFalse(result)

    def test_get_theme_name(self):
        """
        Test get_theme_name function.
        """
        theme = ThemingConfiguration.get_theme_name()
        self.assertEqual(theme, 'default-theme')

    @override_settings(EOX_THEMING_DEFAULT_THEME_NAME='default-theme/inherits/other-theme')
    def test_get_theme_name_tenant(self):
        """
        Test get_theme_name for microsite v0.
        """
        theme = ThemingConfiguration.get_theme_name()
        self.assertEqual(theme, 'other-theme')

    @override_settings(THEME_OPTIONS=[])
    @patch('eox_theming.edxapp_wrapper.config_sources.configuration_helpers')
    def test_comfiguration_helpers_called(self, conf_helpers_mock):
        """ If not found on settings it should use configuration helpers """
        ThemingConfiguration.options('section', 'item', default='Unique_string')
        conf_helpers_mock.get_value.assert_called()

    @patch('eox_theming.configuration.ThemingConfiguration.theming_helpers')
    @patch('eox_theming.configuration.Theme')
    def test_get_parent_theme_is_default(self, theme_mock, helper_mock):
        """
        Test that method return default theme if parent is not defined.
        """
        helper_mock.get_project_root_name.return_value = 'lms'
        helper_mock.get_theme_base_dir.return_value = 'templates'
        ThemingConfiguration.get_parent_or_default_theme()
        theme_mock.assert_called_with(
            name='default-theme',
            project_root='lms',
            theme_dir_name='default-theme',
            themes_base_dir='templates'
        )

    @override_settings(THEME_OPTIONS={'theme': {'parent': 'parent-theme'}})
    @patch('eox_theming.configuration.ThemingConfiguration.theming_helpers')
    @patch('eox_theming.configuration.Theme')
    def test_get_parent_theme(self, theme_mock, helper_mock):
        """
        Test that method return parent theme.
        """
        helper_mock.get_project_root_name.return_value = 'lms'
        helper_mock.get_theme_base_dir.return_value = 'templates'
        ThemingConfiguration.get_parent_or_default_theme()
        theme_mock.assert_called_with(
            name='parent-theme',
            project_root='lms',
            theme_dir_name='parent-theme',
            themes_base_dir='templates'
        )



# -*- coding: utf-8 -*-
"""
Tests for the admin views
"""
from __future__ import absolute_import, unicode_literals

from django.test import TestCase
from django.urls import exceptions, reverse

import eox_theming


class TestUrls(TestCase):
    """ Tests for the urls module """

    def test_urls(self):
        """ Check that url.py loads """
        with self.assertRaises(exceptions.NoReverseMatch):
            reverse('non-existant')


class TestInfoView(TestCase):
    """ Tests for the eox-info page """

    def test_version_is_present(self):
        """ Check that test version is present """
        response = self.client.get('/eox-info')
        self.assertContains(response, eox_theming.__version__)



# -*- coding: utf-8 -*-
"""
Tests for the custom template tags
"""
from __future__ import absolute_import, unicode_literals

from django.template import Context, Template
from django.test import TestCase
from mock import patch


class TestsEoxThemingTags(TestCase):
    """ Tests for the eox_theming template tags """

    @patch('eox_theming.templatetags.eox_theming.ThemingConfiguration.options', return_value='Unique_Return_Value')
    def test_tag_renders(self, _):
        """ Rendering the template must return the value of calling theming.options """
        context = {}
        context = Context(context)

        result = Template("""
            {% load eox_theming %}
            {% theming_options footer item subitem default='my_default' %}
        """).render(context)

        self.assertIn('Unique_Return_Value', result)



"""
This module abstracts the Themes django model classes from the platform core
to prevent a dependency requirement while testing
"""
from importlib import import_module

from django.conf import settings


def get_openedx_site_theme_model():
    """ Get the Open edX site theme model class. """

    backend = import_module(settings.EOX_THEMING_SITE_THEME_BACKEND)

    return backend.get_openedx_site_theme_model()



"""
Common Django settings for eox_theming project.

For more information on this file, see
https://docs.djangoproject.com/en/1.11/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.11/ref/settings/
"""

from __future__ import unicode_literals

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.11/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'secret-key'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'eox_theming'
]

TIME_ZONE = 'UTC'
LANGUAGE_CODE = 'en-us'
USE_TZ = True

# This key needs to be defined so that the check_apps_ready passes and the
# AppRegistry is loaded
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'db.sqlite3',
    }
}


def plugin_settings(settings):
    """
    Set of plugin settings used by the Open Edx platform.
    More info: https://github.com/edx/edx-platform/blob/master/openedx/core/djangoapps/plugins/README.rst
    """
    settings.STATICFILES_FINDERS = [
        'eox_theming.theming.finders.EoxThemeFilesFinder',
    ] + getattr(settings, 'STATICFILES_FINDERS', [])

    try:
        settings.TEMPLATES[0]['OPTIONS']['loaders'][0] = 'eox_theming.theming.template_loaders.EoxThemeTemplateLoader'
    except AttributeError:
        # We must find a way to register this error
        pass

    try:
        eox_configuration_path = 'eox_theming.theming.context_processor.eox_configuration'
        if eox_configuration_path not in settings.TEMPLATES[0]['OPTIONS']['context_processors']:
            settings.TEMPLATES[0]['OPTIONS']['context_processors'].append(eox_configuration_path)
        if eox_configuration_path not in settings.TEMPLATES[1]['OPTIONS']['context_processors']:
            settings.TEMPLATES[1]['OPTIONS']['context_processors'].append(eox_configuration_path)

        settings.DEFAULT_TEMPLATE_ENGINE = settings.TEMPLATES[0]
    except AttributeError:
        # We must find a way to register this error
        pass

    try:
        settings.MIDDLEWARE = [
            'eox_theming.theming.middleware.EoxThemeMiddleware' if 'CurrentSiteThemeMiddleware' in x else x
            for x in settings.MIDDLEWARE
        ]
    except AttributeError:
        # We must find a way to register this error.
        pass

    settings.EOX_THEMING_DEFAULT_THEME_NAME = 'bragi'

    settings.EOX_THEMING_CONFIG_SOURCES = [
        'from_eox_tenant_config_theming',
        'from_eox_tenant_config_lms',
        'from_eox_tenant_microsite_v1',
        'from_eox_tenant_microsite_v0',
        'from_site_config',
        'from_django_settings',
    ]

    settings.EOX_THEMING_BASE_FINDER_BACKEND = 'eox_theming.edxapp_wrapper.backends.j_finders'
    settings.EOX_THEMING_BASE_LOADER_BACKEND = 'eox_theming.edxapp_wrapper.backends.j_loaders'
    settings.EOX_THEMING_SITE_THEME_BACKEND = 'eox_theming.edxapp_wrapper.backends.j_models'
    settings.EOX_THEMING_CONFIGURATION_HELPER_BACKEND = 'eox_theming.edxapp_wrapper.backends.j_configuration_helpers'
    settings.EOX_THEMING_THEMING_HELPER_BACKEND = 'eox_theming.edxapp_wrapper.backends.j_theming_helpers'
    settings.EOX_THEMING_STORAGE_BACKEND = 'eox_theming.edxapp_wrapper.backends.l_storage'
    settings.STATICFILES_STORAGE = 'eox_theming.theming.storage.EoxProductionStorage'

    settings.EOX_THEMING_EDXMAKO_BACKEND = 'eox_theming.edxapp_wrapper.backends.l_mako'



"""
Monkey patches over openedX classes
"""
from collections import OrderedDict

from eox_theming.configuration import ThemingConfiguration
from eox_theming.edxapp_wrapper.theming_helpers import get_theme_class

# This is the original Theme class from edx-platform
# See openedx/core/djangoapps/theming/helpers_dirs.py
Theme = get_theme_class()


class EoxTheme(Theme):
    """
    This is an extension of the openedX Theme class
    """
    def is_current_theme(self):
        """
        Check if the theme instance is the current tenant theme
        """
        return self.name == ThemingConfiguration.get_theme_name()

    def _get_parent_themes(self):
        """
        Get the parent themes for the EoxTheme instance
        """
        parent_themes = OrderedDict()
        # The order of this list is important!
        parent_types = ['parent', 'grandparent']

        for parent in parent_types:
            parent_themes[parent] = None
            parent_theme_name = ThemingConfiguration.options('theme', parent, default=None)
            if parent_theme_name:
                parent_themes[parent] = ThemingConfiguration.get_wrapped_theme(parent_theme_name)

        return parent_themes

    def extend_default_template_dirs(self, dirs):
        """
        Given a list of default template dirs, extend it by prepending the dirs of theme instance and
        its parents
        """
        if not self.is_current_theme():
            return dirs

        dirs = list(dirs)
        parent_themes = self._get_parent_themes()
        dirs_to_prepend = [theme.path / 'templates' for theme in parent_themes.values() if theme]
        dirs_to_prepend.insert(0, self.path / 'templates')

        return dirs_to_prepend + dirs



"""
Simple backend that returns the platform's SiteTheme model
"""
from openedx.core.djangoapps.theming.storage import (  # pylint: disable=import-error
    ThemeManifestFilesMixin,
    ThemePipelineMixin,
    ThemeStorage,
)
from openedx.core.storage import ProductionMixin  # pylint: disable=import-error


def get_theme_storage():
    """Return the ThemeStorage class when called during runtime"""
    return ThemeStorage


def get_themecached_mixin():
    """Return the ThemeCached mixin when called during runtime"""
    return ThemeManifestFilesMixin


def get_themepipeline_mixin():
    """Return the Theme pipeline mixin when called during runtime"""
    return ThemePipelineMixin


def get_production_mixin():
    """Return ProductionMixin when called during runtime"""
    return ProductionMixin



"""
Plugin middlewares
"""
from django.utils.deprecation import MiddlewareMixin

from eox_theming.configuration import ThemingConfiguration
from eox_theming.edxapp_wrapper.models import get_openedx_site_theme_model

SITE_THEME = get_openedx_site_theme_model()


class EoxThemeMiddleware(MiddlewareMixin):
    """
    This Middleware class is required to load a site_theme into the current request.

    We use the site_theme from the core platform still because it behaves well with
    the current implementation of comprehensive themes. It might not be hard requirement later.
    """

    def process_request(self, request):
        """
        Set the request's 'site_theme'
        """
        # TODO: resolve. If the site does not exist, should we create it? or use a placeholder site_id

        theme_name = ThemingConfiguration.get_theme_name()

        if theme_name:
            current_theme = SITE_THEME(
                site_id=1,
                theme_dir_name=theme_name,
            )
            current_theme.id = 1
            request.site_theme = current_theme



"""Backend abstraction for edxmako. """
from importlib import import_module

from django.conf import settings


def get_mako_loader():
    """ Get mako loader """
    backend_function = settings.EOX_THEMING_EDXMAKO_BACKEND
    backend = import_module(backend_function)
    return backend.get_mako_loader()


def get_dynamictemplate_lookup(*args, **kwargs):
    """ Get DynamicTemplateLookup class """
    backend_function = settings.EOX_THEMING_EDXMAKO_BACKEND
    backend = import_module(backend_function)
    return backend.get_dynamictemplate_lookup(*args, **kwargs)


def get_top_level_template_uri(*args, **kwargs):
    """ Get TopLevelTemplateUri """
    backend_function = settings.EOX_THEMING_EDXMAKO_BACKEND
    backend = import_module(backend_function)
    return backend.get_top_level_template_uri(*args, **kwargs)


def get_lookup(*args, **kwargs):
    """ Get lookup from platform code """
    backend_function = settings.EOX_THEMING_EDXMAKO_BACKEND
    backend = import_module(backend_function)
    return backend.get_lookup(*args, **kwargs)


def get_clear_lookups(*args, **kwargs):
    """ Get clear lookups function """
    backend_function = settings.EOX_THEMING_EDXMAKO_BACKEND
    backend = import_module(backend_function)
    return backend.get_clear_lookups(*args, **kwargs)



"""
Theming Storage abstraction used in eox-theming
"""
import os

from django.contrib.staticfiles.finders import find
from django.contrib.staticfiles.storage import StaticFilesStorage
from django.utils.six.moves.urllib.parse import unquote, urlsplit  # pylint: disable=no-name-in-module, import-error
from pipeline.storage import NonPackagingMixin  # pylint: disable=import-error

from eox_theming.configuration import ThemingConfiguration
from eox_theming.edxapp_wrapper.storage import (
    get_production_mixin,
    get_theme_storage,
    get_themecached_mixin,
    get_themepipeline_mixin,
)

OpenedxThemeStorage = get_theme_storage()
ThemeCachedFilesMixin = get_themecached_mixin()
ProductionStorageMixin = get_production_mixin()
ThemePipelineMixin = get_themepipeline_mixin()


class EoxThemeStorage(OpenedxThemeStorage):
    """
    The eox theme storage class is the intervention point to make sure
    that we are loading all the themes statics assets into the platform.

    See: openedx/core/djangoapps/theming/storage.py
    """
    def url(self, name):
        """
        Returns url of the asset, themed url will be returned if the asset is themed otherwise default
        asset url will be returned.

        Args:
            name: name of the asset, e.g. 'images/logo.png'

        Returns:
            url of the asset, e.g. '/static/red-theme/images/logo.png' if current theme is red-theme and logo
            is provided by red-theme otherwise '/static/images/logo.png'
        """
        prefix = ''
        theme = ThemingConfiguration.theming_helpers.get_current_theme()
        parent_theme = ThemingConfiguration.get_parent_or_default_theme()

        # get theme prefix from site address if if asset is accessed via a url
        if theme:
            prefix = theme.theme_dir_name

        # get theme prefix from storage class, if asset is accessed during collectstatic run
        elif self.prefix:
            prefix = self.prefix

        # join theme prefix with asset name if theme is applied and themed asset exists
        if prefix and self.themed(name, prefix):
            return super(EoxThemeStorage, self).url(name)

        if parent_theme and self.themed(name, parent_theme.theme_dir_name):
            name = os.path.join(parent_theme.theme_dir_name, name)
            return super(EoxThemeStorage, self).url(name)

        grandparent_name = ThemingConfiguration.options('theme', 'grandparent', default=None)
        if grandparent_name:
            grandparent_theme = ThemingConfiguration.get_wrapped_theme(grandparent_name)
            if grandparent_theme and self.themed(name, grandparent_theme.theme_dir_name):
                name = os.path.join(grandparent_theme.theme_dir_name, name)

        return super(EoxThemeStorage, self).url(name)


class AbsoluteUrlAssetsMixin:
    """
    Mixin that overrides the url method on storages
    """
    def url(self, name):
        """
        Return url of the asset.
        If the asset name is an absolute url, just return the asset name
        """
        if name.startswith("https://") or name.startswith("http://"):
            return name

        return super(AbsoluteUrlAssetsMixin, self).url(name)


class EoxProductionStorage(
        AbsoluteUrlAssetsMixin,
        ProductionStorageMixin,
        EoxThemeStorage,
        StaticFilesStorage
):
    """
    This class combines Django's StaticFilesStorage class with several mixins
    that provide additional functionality. We use this version on production.
    """
    def _processed_asset_name(self, name):
        """
        Returns either a themed or unthemed version of the given asset name,
        depending on several factors.

        See the class docstring for more info.
        """
        theme = ThemingConfiguration.theming_helpers.get_current_theme()
        if theme and theme.theme_dir_name not in name:
            # during server run, append theme name to the asset name if it is not already there
            # this is ensure that correct hash is created and default asset is not always
            # used to create hash of themed assets.
            name = os.path.join(theme.theme_dir_name, name)
        parsed_name = urlsplit(unquote(name))
        clean_name = parsed_name.path.strip()
        asset_name = name
        if not self.exists(clean_name):
            # if themed asset does not exists then use default asset
            theme_name = name.split("/", 1)[0]
            # verify that themed asset was accessed
            if theme_name in [theme.theme_dir_name for theme in ThemingConfiguration.theming_helpers.get_themes()]:
                asset_name = "/".join(name.split("/")[1:])
        elif theme and theme.theme_dir_name in asset_name:
            return asset_name

        # Try the same with default theme
        parent_theme = ThemingConfiguration.get_parent_or_default_theme()

        if theme and parent_theme.name == theme.name:
            return asset_name

        theme = parent_theme
        if theme and theme.theme_dir_name not in asset_name:
            # during server run, append theme name to the asset name if it is not already there
            # this is ensure that correct hash is created and default asset is not always
            # used to create hash of themed assets.
            name = os.path.join(theme.theme_dir_name, asset_name)
        parsed_name = urlsplit(unquote(name))
        clean_name = parsed_name.path.strip()
        asset_name = name
        if not self.exists(clean_name):
            # if themed asset does not exists then use default asset
            theme = name.split("/", 1)[0]
            # verify that themed asset was accessed
            if theme in [theme.theme_dir_name for theme in ThemingConfiguration.theming_helpers.get_themes()]:
                asset_name = "/".join(name.split("/")[1:])

        return asset_name

    @staticmethod
    def get_themed_packages(prefix, packages):
        """
        Method overriding to avoid including non themed assets during the collect static command,
        so the themes assets are loaded in the correct order for the sites.

        Update paths with the themed assets,
        Args:
            prefix: theme prefix for which to update asset paths e.g. 'red-theme', 'edx.org' etc.
            packages: packages to update

        Returns: list of updated paths and a boolean indicating whether any path was path or not
        """
        themed_packages = {}
        for name in packages:
            # collect source file names for the package
            source_files = []
            for path in packages[name].get('source_filenames', []):
                # if themed asset exists use that, otherwise do not include it.
                if find(os.path.join(prefix, path)):
                    source_files.append(os.path.join(prefix, path))

            if source_files:
                themed_packages[name] = {
                    'output_filename': os.path.join(prefix, packages[name].get('output_filename', '')),
                    'source_filenames': source_files,
                }
        return themed_packages


class EoxDevelopmentStorage(
        AbsoluteUrlAssetsMixin,
        NonPackagingMixin,
        ThemePipelineMixin,
        EoxThemeStorage,
        StaticFilesStorage
):
    """
    This class combines Django's StaticFilesStorage class with several mixins
    that provide additional functionality. We use this version for development,
    so that we can skip packaging and optimization.
    """



""" Backend abstraction for theming helpers tests. """
from eox_theming.test_utils import TestThemingHelpersDirs


def get_theming_helpers():
    """ Backend abstraction used on the tests """
    return object


def get_theming_helpers_dirs():
    """ Backend to get the theming helpers dirs on the tests. """
    return TestThemingHelpersDirs()


def get_theme_class():
    """ Backend to get the Theme class on the tests. """
    return object



"""
Simple backend that returns the platform's ThemeFilesFinder class
"""
from openedx.core.djangoapps.theming.finders import ThemeFilesFinder  # pylint: disable=import-error


def get_openedx_theme_finder():
    """Return the Finder class when called during runtime"""
    return ThemeFilesFinder



"""
Manages the diferent access, storage locations and methods used to read the configuration of a theme
"""
import logging

from django.conf import settings

from eox_theming.edxapp_wrapper import config_sources
from eox_theming.edxapp_wrapper.theming_helpers import get_theme_class, get_theming_helpers

LOG = logging.getLogger(__name__)
Theme = get_theme_class()


class ThemingConfiguration:
    """
    This class must handle all the calls to the diferential settings a theme will be allowed to use
    """
    source_functions = [getattr(config_sources, name) for name in settings.EOX_THEMING_CONFIG_SOURCES]
    theming_helpers = get_theming_helpers()

    @classmethod
    def options(cls, *args, **kwargs):
        """Main method for accessing the current configuration for the theme"""
        value = None

        for source in cls.source_functions:
            try:
                value = source(*args)
                if value is not None:
                    break
            except Exception as exc:  # pylint: disable=broad-except
                LOG.warning('Found an error reading %s from source %s. Trace: %s',
                            '.'.join(args), source.__name__, exc)
                continue

        if value is None:
            value = kwargs.pop('default', None)

        return value

    @classmethod
    def get_theme_name(cls):
        """
        Get the current theme name
        Returns:
            (str): Theme name associated to the request.
        """
        theme_name = cls.options('theme', 'name', default=None)

        if not theme_name:
            theme_name = cls.options('template_dir', default=settings.EOX_THEMING_DEFAULT_THEME_NAME)

        if theme_name:
            theme_name = theme_name.split('/')[-1]

        return theme_name.lower()

    @classmethod
    def get_parent_or_default_theme(cls):
        """ Get parent theme of the current request. """
        parent_theme_name = cls.options('theme', 'parent', default=None)
        if not parent_theme_name:
            parent_theme_name = settings.EOX_THEMING_DEFAULT_THEME_NAME
        return cls.get_wrapped_theme(parent_theme_name)

    @classmethod
    def get_wrapped_theme(cls, theme_name):
        """ Get theme based on the input name. """
        theme_name = theme_name.lower()
        try:
            return Theme(
                name=theme_name,
                theme_dir_name=theme_name,
                themes_base_dir=cls.theming_helpers.get_theme_base_dir(theme_name),
                project_root=cls.theming_helpers.get_project_root_name()
            )
        except ValueError as error:
            # Log exception message and return None, so that open source theme is used instead
            LOG.exception('Theme not found in any of the themes dirs. [%s]', error)
        return None



"""
Production Django settings for eox_theming project.
"""

from __future__ import unicode_literals


def plugin_settings(settings):
    """
    Set of plugin settings used by the Open Edx platform.
    More info: https://github.com/edx/edx-platform/blob/master/openedx/core/djangoapps/plugins/README.rst
    """
    settings.EOX_THEMING_DEFAULT_THEME_NAME = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_THEMING_DEFAULT_THEME_NAME',
        settings.EOX_THEMING_DEFAULT_THEME_NAME
    )



"""
Module to test patched modules
"""
import operator
from collections import OrderedDict
# Forward compatibility for Python 3
from functools import reduce  # pylint: disable=redefined-builtin, useless-suppression

import six
from django.test import TestCase
from mock import patch
from path import Path

from eox_theming.theming.patches import EoxTheme


class TestEoxTheme(TestCase):
    """
    Tests for EoxTheme class which extends the Theme openedx class
    """
    def setUp(self):
        """
        Test Case Setup
        """
        theme1 = EoxTheme()
        theme1.name = 'mytheme'
        theme1.theme_dir_name = 'mytheme'
        theme1.themes_base_dir = Path('/my/path/to/theme')
        theme1.project_root = 'lms'
        theme1.path = (theme1.themes_base_dir
                       / theme1.theme_dir_name
                       / theme1.project_root)
        self.theme1 = theme1

        theme_parent = EoxTheme()
        theme_parent.name = 'mythemeparent'
        theme_parent.theme_dir_name = 'mythemeparent'
        theme_parent.themes_base_dir = Path('/my/path/to/themeparent')
        theme_parent.project_root = 'lms'
        theme_parent.path = (theme_parent.themes_base_dir
                             / theme_parent.theme_dir_name
                             / theme_parent.project_root)
        self.theme_parent = theme1

        theme_grandparent = EoxTheme()
        theme_grandparent.name = 'mythemegrandparent'
        theme_grandparent.theme_dir_name = 'mythemegrandparent'
        theme_grandparent.themes_base_dir = Path('/my/path/to/themegrandparent')
        theme_grandparent.project_root = 'lms'
        theme_grandparent.path = (theme_grandparent.themes_base_dir
                                  / theme_grandparent.theme_dir_name
                                  / theme_grandparent.project_root)
        self.theme_grandparent = theme1

    @patch('eox_theming.configuration.ThemingConfiguration.get_theme_name')
    def test_is_current_theme(self, get_theme_name_mock):
        """
        Check the is_current_theme method
        """
        # Test the case when the theme is the current theme
        get_theme_name_mock.return_value = 'mytheme'
        result = self.theme1.is_current_theme()
        get_theme_name_mock.assert_called_once()
        self.assertTrue(result)

        # Test the case when the theme is the current theme
        get_theme_name_mock.reset_mock()
        get_theme_name_mock.return_value = 'mynewtheme'
        result = self.theme1.is_current_theme()
        get_theme_name_mock.assert_called_once()
        self.assertFalse(result)

    @patch('eox_theming.configuration.ThemingConfiguration.get_wrapped_theme')
    @patch('eox_theming.configuration.ThemingConfiguration.options')
    def test_get_parent_themes(self, options_mock, get_wrapped_theme_mock):
        """
        Check the _get_parent_themes method
        """
        theme_options = {
            'theme': {
                'parent': 'mythemeparent',
                'grandparent': 'mythemegrandparent'
            }
        }

        themes_set = {
            'mythemeparent': self.theme_parent,
            'mythemegrandparent': self.theme_grandparent
        }

        def options_side_effect(*args, **kwargs):
            """
            ThemingConfiguration options side effect
            """
            if args:
                try:
                    value = reduce(operator.getitem, args, theme_options)
                except (AttributeError, KeyError):
                    value = None

            if value is None:
                value = kwargs.pop('default', None)

            return value

        def get_wrapped_theme_side_effect(name):
            """
            ThemingConfiguration get_wrapped_theme side effect
            """
            return themes_set.get(name, None)

        options_mock.side_effect = options_side_effect
        get_wrapped_theme_mock.side_effect = get_wrapped_theme_side_effect

        res = self.theme1._get_parent_themes()  # pylint: disable=protected-access
        options_mock.assert_called()
        get_wrapped_theme_mock.assert_called()
        self.assertListEqual(list(res.keys()), ['parent', 'grandparent'])

    @patch('eox_theming.theming.patches.EoxTheme._get_parent_themes')
    @patch('eox_theming.theming.patches.EoxTheme.is_current_theme')
    def test_extend_no_current_theme(self, is_current_theme_mock, get_parent_themes_mock):
        """
        Test that dirs are not modified when the theme is not the current theme
        """
        dirs = ['/mydir/fisrt', '/mydir/second']
        is_current_theme_mock.return_value = False
        res = self.theme1.extend_default_template_dirs(dirs)
        is_current_theme_mock.assert_called_once()
        get_parent_themes_mock.assert_not_called()
        self.assertListEqual(res, dirs)

    @patch('eox_theming.theming.patches.EoxTheme._get_parent_themes')
    @patch('eox_theming.theming.patches.EoxTheme.is_current_theme')
    def test_extend_template_dirs(self, is_current_theme_mock, get_parent_themes_mock):
        """
        Test that default template dirs are extended correctly
        """
        dirs = ['/mydir/fisrt', '/mydir/second']
        is_current_theme_mock.return_value = True

        parent_themes = OrderedDict()
        parent_themes['parent'] = self.theme_parent
        parent_themes['grandparent'] = self.theme_grandparent
        get_parent_themes_mock.return_value = parent_themes

        res = self.theme1.extend_default_template_dirs(dirs)
        is_current_theme_mock.assert_called_once()
        get_parent_themes_mock.assert_called_once()
        self.assertEqual(six.text_type(res[0]), six.text_type(self.theme1.path) + '/templates')



""" Backend abstraction of configuration helpers. """
from openedx.core.djangoapps.site_configuration import helpers as configuration_helpers  # pylint: disable=import-error


def get_configuration_helper():
    """ Backend to get the configuration helper. """
    return configuration_helpers



# -*- coding: utf-8 -*-
# -*- coding: utf-8 -*-
"""
Tests for the template loaders
"""
from __future__ import absolute_import, unicode_literals

from django.test import TestCase
from mock import mock, patch
from path import Path

from eox_theming.theming.template_loaders import EoxThemeFilesystemLoader


class TestsEoxFSLoader(TestCase):
    """ Tests for the eox_theming filesystem loader """

    @patch('eox_theming.configuration.ThemingConfiguration.theming_helpers')
    @patch('eox_theming.configuration.ThemingConfiguration.get_parent_or_default_theme')
    def test_returned_default_theme_template_sources(self, parent_mock, helper_mock):
        """
        Test if it is returned the template dirs of the default theme.
        """
        theme = mock.Mock()
        theme.theme_dir_name = 'bragi'
        theme.name = 'bragi'
        helper_mock.get_current_theme.return_value = theme
        helper_mock.get_project_root_name.return_value = 'lms'

        parent_theme = mock.Mock()
        parent_theme.theme_dir_name = 'default-theme'
        parent_theme.name = 'default-theme'
        parent_theme.template_dirs = Path('/ednx/var/themes/edx-platform/default-theme')
        parent_mock.return_value = parent_theme

        parent_theme_sources = EoxThemeFilesystemLoader.get_parent_theme_template_sources()
        self.assertEqual(parent_theme_sources, parent_theme.template_dirs)

    @patch('eox_theming.configuration.ThemingConfiguration.theming_helpers')
    @patch('eox_theming.theming.template_loaders.EoxThemeFilesystemLoader.get_grandparent_theme_template_sources')
    @patch('eox_theming.theming.template_loaders.EoxThemeFilesystemLoader.get_parent_theme_template_sources')
    def test_returned_theme_template_sources(self, parent_mock, grandparent_mock, helper_mock):
        """
        Test if get_theme_template_sources method returns the template directories in the correct order.
        """
        theme = mock.Mock()
        theme.theme_dir_name = 'bragi'
        theme.name = 'bragi'
        theme.template_dirs = [Path('/ednx/var/themes/edx-platform/bragi')]
        helper_mock.get_current_theme.return_value = theme
        helper_mock.get_project_root_name.return_value = 'lms'

        parent_theme_template_dirs = [Path('/ednx/var/themes/edx-platform/parent-theme')]
        grandparent_theme_template_dirs = [Path('/ednx/var/themes/edx-platform/grandparent-theme')]
        template_dirs = theme.template_dirs + parent_theme_template_dirs + grandparent_theme_template_dirs

        parent_mock.return_value = parent_theme_template_dirs
        grandparent_mock.return_value = grandparent_theme_template_dirs

        theme_template_sources = EoxThemeFilesystemLoader.get_theme_template_sources()
        self.assertEqual(theme_template_sources, template_dirs)



"""
Simple abstraction for tests
"""


def get_openedx_theme_loader():
    """Abstraction for tests"""
    return object


def get_theme_filesystem_loader():
    """Abstraction for tests"""
    return object



"""
Test Django settings for eox_theming project.
"""

from __future__ import unicode_literals

from .common import *  # pylint: disable=wildcard-import


class SettingsClass:
    """ dummy settings class """


SETTINGS = SettingsClass()
plugin_settings(SETTINGS)
vars().update(SETTINGS.__dict__)
INSTALLED_APPS = vars().get('INSTALLED_APPS', [])
TEST_INSTALLED_APPS = [
    'django.contrib.sites',
]
for app in TEST_INSTALLED_APPS:
    if app not in INSTALLED_APPS:
        INSTALLED_APPS.append(app)

ROOT_URLCONF = 'eox_theming.urls'
ALLOWED_HOSTS = ['*']

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {},
    },
]

EOX_THEMING_DEFAULT_THEME_NAME = 'default-theme'

EOX_THEMING_CONFIGURATION_HELPER_BACKEND = 'eox_theming.edxapp_wrapper.backends.j_configuration_helpers_tests'
EOX_THEMING_THEMING_HELPER_BACKEND = 'eox_theming.edxapp_wrapper.backends.j_theming_helpers_tests'
EOX_THEMING_EDXMAKO_BACKEND = 'eox_theming.edxapp_wrapper.backends.j_mako_tests'
EOX_THEMING_BASE_LOADER_BACKEND = 'eox_theming.edxapp_wrapper.backends.j_loaders_tests'


def plugin_settings(settings):  # pylint: disable=function-redefined
    """
    For the platform tests, we want everything to be disabled
    """
    theme_template_loader = 'openedx.core.djangoapps.theming.template_loaders.ThemeTemplateLoader'
    settings.TEMPLATES[0]['OPTIONS']['loaders'][0] = theme_template_loader

    try:
        settings.MIDDLEWARE = [
            'openedx.core.djangoapps.theming.middleware.CurrentSiteThemeMiddleware' if 'EoxThemeMiddleware' in x else x
            for x in settings.MIDDLEWARE
        ]
    except AttributeError:
        pass

    settings.STATICFILES_STORAGE = 'openedx.core.storage.ProductionStorage'

    settings.STATICFILES_FINDERS = [
        x for x in settings.STATICFILES_FINDERS if 'EoxThemeFilesFinder' not in x
    ]



"""
This module abstracts the File Finder class from the platform core to prevent a dependency requirement while testing
"""
from importlib import import_module

from django.conf import settings


def get_openedx_theme_finder():
    """ Get the Open edX Theme Finder class. """

    backend = import_module(settings.EOX_THEMING_BASE_FINDER_BACKEND)

    return backend.get_openedx_theme_finder()



"""Backend abstraction for theming helpers. """
from importlib import import_module

from django.conf import settings


def get_theming_helpers(*args, **kwargs):
    """ Get theming helper module """
    backend_function = settings.EOX_THEMING_THEMING_HELPER_BACKEND
    backend = import_module(backend_function)
    return backend.get_theming_helpers(*args, **kwargs)


def get_theming_helpers_dirs(*args, **kwargs):
    """ Get theming helper dirs module """
    backend_function = settings.EOX_THEMING_THEMING_HELPER_BACKEND
    backend = import_module(backend_function)
    return backend.get_theming_helpers_dirs(*args, **kwargs)


def get_theme_class(*args, **kwargs):
    """ Get theme class """
    backend_function = settings.EOX_THEMING_THEMING_HELPER_BACKEND
    backend = import_module(backend_function)
    return backend.get_theme_class(*args, **kwargs)



"""
Setup file for eox_theming Django plugin.
"""

from __future__ import print_function

import os
import re

from setuptools import setup


def get_version(*file_paths):
    """
    Extract the version string from the file at the given relative path fragments.
    """
    filename = os.path.join(os.path.dirname(__file__), *file_paths)
    version_file = open(filename).read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')


VERSION = get_version('eox_theming', '__init__.py')


setup(
    name='eox-theming',
    version=VERSION,
    description='Open edX Theming Plugin',
    author='eduNEXT',
    author_email='contact@edunext.co',
    packages=[
        'eox_theming'
    ],
    include_package_data=True,
    install_requires=[],
    zip_safe=False,
    entry_points={
        "lms.djangoapp": [
            'eox_theming = eox_theming.apps:EoxThemingConfig',
        ],
    }
)



"""
App configuration for eox_theming.
"""

from __future__ import unicode_literals

from django.apps import AppConfig
from django.conf import settings


class EoxThemingConfig(AppConfig):
    """
    Open edX Theming Plugin configuration.
    """
    name = 'eox_theming'
    verbose_name = 'Open edX Theming Plugin'

    plugin_app = {
        'url_config': {
            'lms.djangoapp': {
                'namespace': 'eox_theming',
                'regex': r'^eox-theming/',
                'relative_path': 'urls',
            },
            'cms.djangoapp': {
                'namespace': 'eox_theming',
                'regex': r'^eox-theming/',
                'relative_path': 'urls',
            }
        },
        'settings_config': {
            'lms.djangoapp': {
                'common': {'relative_path': 'settings.common'},
                'test': {'relative_path': 'settings.test'},
                'production': {'relative_path': 'settings.production'},
                'devstack': {'relative_path': 'settings.devstack'},
            },
            'cms.djangoapp': {
                'common': {'relative_path': 'settings.common'},
                'test': {'relative_path': 'settings.test'},
                'production': {'relative_path': 'settings.production'},
            },
        }
    }

    def ready(self):
        """
        Method to perform actions after apps registry is ended
        Setup mako lookup directories.
        See: common.djangoapps.edxmako.apps.py
        """
        from eox_theming.theming.paths import add_lookup, clear_lookups  # pylint: disable=import-outside-toplevel
        for backend in settings.TEMPLATES:
            if 'edxmako' not in backend['BACKEND']:
                continue
            namespace = backend['OPTIONS'].get('namespace', 'main')
            directories = backend['DIRS']
            clear_lookups(namespace)
            for directory in directories:
                add_lookup(namespace, directory)

        self.apply_patches()

    def apply_patches(self):
        """
        Method to apply monkey patches over openedX classes
        """
        from eox_theming.edxapp_wrapper.theming_helpers import get_theming_helpers_dirs, get_theming_helpers    # pylint: disable=import-outside-toplevel
        from eox_theming import configuration    # pylint: disable=import-outside-toplevel
        from eox_theming.theming.patches import EoxTheme    # pylint: disable=import-outside-toplevel

        theming_helpers = get_theming_helpers()
        theming_helpers_dirs = get_theming_helpers_dirs()
        theming_helpers.Theme = EoxTheme
        theming_helpers_dirs.Theme = EoxTheme
        configuration.Theme = EoxTheme



"""
This module abstracts the Template Loader class from the
platform core toprevent a dependency requirement while testing
"""
from importlib import import_module

from django.conf import settings


def get_openedx_theme_loader():
    """ Get the Open edX template loader class. """
    backend = import_module(settings.EOX_THEMING_BASE_LOADER_BACKEND)
    return backend.get_openedx_theme_loader()


def get_theme_filesystem_loader():
    """ Get the Open edX filesystem loader class. """
    backend = import_module(settings.EOX_THEMING_BASE_LOADER_BACKEND)
    return backend.get_theme_filesystem_loader()



""" Backend abstraction for theming helpers. """
from openedx.core.djangoapps.theming import helpers as theming_helpers  # pylint: disable=import-error
from openedx.core.djangoapps.theming import helpers_dirs as theming_helpers_dirs  # pylint: disable=import-error


def get_theming_helpers():
    """ Backend to get the theming helpers. """
    return theming_helpers


def get_theming_helpers_dirs():
    """ Backend to get the theming helpers dirs. """
    return theming_helpers_dirs


def get_theme_class():
    """ Backend to get the Theme class. """
    return theming_helpers_dirs.Theme



"""
This function gets called during every request by the
context processor to return all the custom scripts for a specific path.
"""
import logging
import re

import six

from eox_theming.configuration import ThemingConfiguration

logger = logging.getLogger(__name__)

common_attributes = {'type': {'options': ['inline', 'external']},
                     'media_type': {'options': ['module', 'text/javascript'], 'default': 1}}


def validate_script_attributes(values):
    """
    Validate common attributes for external and inline scripts.
    """
    script_attributes = {}

    # Validate common attributes for external and inline scripts
    for key, attr_values in common_attributes.items():

        value = values.get(key)

        try:
            if value is not None:
                attr_values['options'].index(value.lower())

                script_attributes[key] = value.lower()
            else:
                default = attr_values['default']

                script_attributes[key] = attr_values['options'][default]

        except Exception:  # pylint: disable=broad-except
            logger.error("Script could not get loaded. '%s' attribute is missing or is an invalid option.", key)

            return None

    # Validate according to the script type
    attr = 'src'
    if script_attributes['type'] == 'inline':
        attr = 'content'

    script_attributes = check_attribute(values, script_attributes, attr)

    return script_attributes


def check_attribute(values, script_attributes, attribute):
    """
    Validate the existence of an specific attribute.
    """
    try:
        script_attributes[attribute] = values[attribute]
    except Exception:  # pylint: disable=broad-except
        logger.error("Script could not get loaded. '%s' attribute is missing.", attribute)

        return None

    return script_attributes


def process_scripts(path):
    """
    Process and loads all the extra scripts for the template
    rendered during the request.
    """
    scripts = ThemingConfiguration.options('scripts', default={})

    for regex, values in six.iteritems(scripts):

        regex_path_match = re.compile(regex)

        if regex_path_match.match(path):

            scripts = []

            for script in values:

                validated_script = validate_script_attributes(script)

                if validated_script:

                    scripts.append(validated_script)

            if scripts:
                return scripts

    return None



"""
Production Django settings for eox_theming project.
"""

from __future__ import unicode_literals


def plugin_settings(settings):
    """
    Set of plugin settings used by the Open Edx platform.
    More info: https://github.com/edx/edx-platform/blob/master/openedx/core/djangoapps/plugins/README.rst
    """
    settings.STATICFILES_FINDERS = [
        'eox_theming.theming.finders.EoxThemeFilesFinder',
    ] + settings.STATICFILES_FINDERS

    settings.STATICFILES_STORAGE = 'eox_theming.theming.storage.EoxDevelopmentStorage'



"""
eox_theming URL Configuration
"""
from django.conf.urls import include, url

urlpatterns = [
    url(r'', include(('eox_theming.management.urls', 'eox_theming'), namespace='eox-theming-management')),
]
